from marshmallow import Schema, fields


class EmptySchema(Schema):
    """
    Dummy schema for making empty requests without swagger validation errors.
    """
    _ = fields.Boolean(required=True, dump_only=True)



from flask import request
from microcosm.api import defaults


X_REQUEST = "X-Request"


def context_wrapper(include_header_prefix):
    def retrieve_context():
        return {
            header: value
            for header, value in request.headers.items()
            if header.startswith(include_header_prefix)
        }

    return retrieve_context


@defaults(
    include_header_prefix=X_REQUEST,
)
def configure_request_context(graph):
    """
    Configure the flask context function which controls what data you want to associate
    with your flask request context, e.g. headers, request body/response.

    Usage:
        graph.request_context()
    """

    include_header_prefix = graph.config.request_context.include_header_prefix
    return context_wrapper(include_header_prefix)



"""
Compare resources from two YAML files.

Useful for validating that a sync in/out retains the same state.

"""
from argparse import ArgumentParser

from yaml import load_all


def to_dict(path):
    """
    Normalize each resource as a dictionary for smarter comparison.

    Relies on link sorting performed by the pull script.

    """
    with open(path) as file_:
        return {
            key: value
            for dct in load_all(file_)
            for key, value in dct.items()
        }


def main():
    parser = ArgumentParser()
    parser.add_argument("left")
    parser.add_argument("right")
    args = parser.parse_args()

    left, right = to_dict(args.left), to_dict(args.right)

    left_keys = set(left.keys())
    right_keys = set(right.keys())
    if left_keys != right_keys:
        print "Only in left:"  # noqa
        for key in left_keys - right_keys:
            print " - {}".format(key)  # noqa
        print "Only in right:"  # noqa
        for key in right_keys - left_keys:
            print " - {}".format(key)  # noqa
        exit(1)

    for key in left.keys():
        if left[key] != right[key]:
            print "Different values for: {}".format(key)  # noqa
            print "-" * 20  # noqa
            print "Left:"  # noqa
            print left[key]  # noqa
            print "Right:"  # noqa
            print right[key]  # noqa
            exit(1)

if __name__ == '__main__':
    main()



"""
Hamcrest matching support for JSON responses.

"""
from json import dumps, loads
from hamcrest.core.base_matcher import BaseMatcher


def prettify(value):
    return dumps(
        value,
        sort_keys=True,
        indent=4,
        separators=(',', ': '),
    )


class JSON(object):
    """
    Dictionary wrapper with JSON pretty-printing for Hamcrest's description.

    """
    def __init__(self, dct):
        self.dct = dct

    def __getitem__(self, key):
        return self.dct[key]

    def get(self, key):
        return self.dct.get(key)

    def describe_to(self, description):
        description.append(prettify(self.dct))


def json_for(value):
    if not isinstance(value, (dict, list)):
        value = loads(value)
    return JSON(value)


class JSONMatcher(BaseMatcher):
    """
    Hamcrest matcher of a JSON encoded resource.

    Subclasses must define `_matcher` and invoke `assert_that` within a request
    context to ensure that Flask's `url_for` can be resolved.

    Example:

        with graph.app.test_request_context():
           assert_that(json(response.data), matches_myresource(expected))

    """
    def __init__(self, resource):
        self.resource = resource
        self.schema = self.schema_class()
        self.expected = self.schema.dump(self.resource).data

    @property
    def schema_class(self):
        raise NotImplementedError

    def describe_to(self, description):
        description.append_text("expected {}".format(prettify(self.expected)))
